@@ -1,26 +1,19 @@
 """
-This module provides functions and metadata that can be used to convert the
-hourly Global Drifter Program (GDP) data to a ``clouddrift.RaggedArray`` instance.
+This module provides functions and metadata to convert the Global Drifter
+Program (GDP) data to a ``clouddrift.RaggedArray`` instance. The functions
+defined in this module are common to both hourly (``clouddrift.adapters.gdp1h``)
+and six-hourly (``clouddrift.adapters.gdp6h``) GDP modules.
 """
 
-from ..dataformat import RaggedArray
 import numpy as np
 import pandas as pd
 from datetime import datetime
 import xarray as xr
 import urllib.request
-import concurrent.futures
-import re
-import tempfile
-from tqdm import tqdm
-from typing import Optional
 import os
 import warnings
 
 GDP_VERSION = "2.00"
-GDP_DATA_URL = "https://www.aoml.noaa.gov/ftp/pub/phod/lumpkin/hourly/v2.00/netcdf/"
-GDP_FILENAME_PATTERN = "drifter_{id}.nc"
-GDP_TMP_PATH = os.path.join(tempfile.gettempdir(), "clouddrift", "gdp")
 
 GDP_COORDS = [
     "ids",
@@ -90,17 +83,18 @@ GDP_DATA = [
 
 
 def parse_directory_file(filename: str) -> pd.DataFrame:
-    """Read a directory file which contains metadata of drifters' releases.
+    """Read a GDP directory file that contains metadata of drifter releases.
 
-    Due to naming of these files, a manual update of the last file name after an
-    update of the dataset is needed.
+    Parameters
+    ----------
+    filename : str
+        Name of the directory file to parse.
 
-    Args:
-        filename (str): Name of the directory file
-    Returns:
-        pd.DataFrame: Sorted list of drifters
+    Returns
+    -------
+    df : pd.DataFrame
+        List of drifters from a single directory file as a pandas DataFrame.
     """
-
     GDP_DIRECTORY_FILE_URL = "https://www.aoml.noaa.gov/ftp/pub/phod/buoydata/"
     df = pd.read_csv(
         os.path.join(GDP_DIRECTORY_FILE_URL, filename), delimiter="\s+", header=None
@@ -116,116 +110,98 @@ def parse_directory_file(filename: str) -> pd.DataFrame:
         "WMO_number",
         "program_number",
         "buoys_type",
-        "Deployment_date",
-        "Deployment_lat",
-        "Deployment_lon",
+        "Start_date",
+        "Start_lat",
+        "Start_lon",
         "End_date",
         "End_lat",
         "End_lon",
         "Drogue_off_date",
         "death_code",
     ]
-    for t in ["Deployment_date", "End_date", "Drogue_off_date"]:
+    for t in ["Start_date", "End_date", "Drogue_off_date"]:
         df[t] = pd.to_datetime(df[t], format="%Y/%m/%d %H:%M", errors="coerce")
 
     return df
 
 
 def get_gdp_metadata() -> pd.DataFrame:
-    """Download and parse GDP metadata and return it as a Pandas DataFrame."""
+    """Download and parse GDP metadata and return it as a Pandas DataFrame.
 
-    directory_file_names = [
-        "dirfl_1_5000.dat",
-        "dirfl_5001_10000.dat",
-        "dirfl_10001_15000.dat",
-        "dirfl_15001_jul22.dat",
-    ]
-    df = pd.concat([parse_directory_file(f) for f in directory_file_names])
-    df.sort_values(["Deployment_date"], inplace=True, ignore_index=True)
+    Returns
+    -------
+    df : pd.DataFrame
+        Sorted list of drifters as a pandas DataFrame.
+    """
+    directory_file_pattern = "dirfl_{low}_{high}.dat"
+
+    dfs = []
+    start = 1
+    while True:
+        name = directory_file_pattern.format(low=start, high=start + 4999)
+        try:
+            dfs.append(parse_directory_file(name))
+            start += 5000
+        except:
+            break
+
+    name = directory_file_pattern.format(low=start, high="current")
+    dfs.append(parse_directory_file(name))
+
+    df = pd.concat(dfs)
+    df.sort_values(["Start_date"], inplace=True, ignore_index=True)
     return df
 
 
 def order_by_date(df: pd.DataFrame, idx: list[int]) -> np.ndarray[int]:
-    """From the previously sorted directory files DataFrame, return the drifter
-    indices sorted by their end date.
+    """From the previously sorted DataFrame of directory files, return the
+    unique set of drifter IDs sorted by their start date (the date of the first
+    quality-controlled data point).
 
-    Args:
-        idx [list]: List of drifters to include in the ragged array
-    Returns:
-        idx [list]: Sorted list of drifters
+    Parameters
+    ----------
+    idx : list
+        List of drifters to include in the ragged array
+
+    Returns
+    -------
+    idx : list
+        Unique set of drifter IDs sorted by their start date.
     """
     return df.ID[np.where(np.in1d(df.ID, idx))[0]].values
 
 
 def fetch_netcdf(url: str, file: str):
-    """Download and save the file from the given url, if not already downloaded."""
-    if not os.path.isfile(file):
-        req = urllib.request.urlretrieve(url, file)
+    """Download and save the file from the given url, if not already downloaded.
 
-
-def download(drifter_ids: list = None, n_random_id: int = None):
-    """Download individual NetCDF files from the AOML server.
-
-    :param drifter_ids [list]: list of drifter to retrieve (Default: all)
-    :param n_random_id [int]: randomly select n drifter NetCDF files
-    :return drifters_ids [list]: list of retrived drifter
+    Parameters
+    ----------
+    url : str
+        URL from which to download the file.
+    file : str
+        Name of the file to save.
     """
-
-    # Create a temporary directory if doesn't already exists.
-    os.makedirs(GDP_TMP_PATH, exist_ok=True)
-
-    # retrieve all drifter ID numbers
-    if drifter_ids is None:
-        urlpath = urllib.request.urlopen(GDP_DATA_URL)
-        string = urlpath.read().decode("utf-8")
-        pattern = re.compile("drifter_[0-9]*.nc")
-        filelist = pattern.findall(string)
-        drifter_ids = np.unique([int(f.split("_")[-1][:-3]) for f in filelist])
-
-    # retrieve only a subset of n_random_id trajectories
-    if n_random_id:
-        if n_random_id > len(drifter_ids):
-            warnings.warn(
-                f"Retrieving all listed trajectories because {n_random_id} is larger than the {len(drifter_ids)} listed trajectories."
-            )
-        else:
-            rng = np.random.RandomState(42)
-            drifter_ids = sorted(rng.choice(drifter_ids, n_random_id, replace=False))
-
-    with concurrent.futures.ThreadPoolExecutor() as executor:
-        # create list of urls and paths
-        urls = []
-        files = []
-        for i in drifter_ids:
-            file = GDP_FILENAME_PATTERN.format(id=i)
-            urls.append(os.path.join(GDP_DATA_URL, file))
-            files.append(os.path.join(GDP_TMP_PATH, file))
-
-        # parallel retrieving of individual netCDF files
-        list(
-            tqdm(
-                executor.map(fetch_netcdf, urls, files),
-                total=len(files),
-                desc="Downloading files",
-                ncols=80,
-            )
-        )
-
-    # Download the metadata so we can order the drifter IDs by end date.
-    gdp_metadata = get_gdp_metadata()
-
-    return order_by_date(gdp_metadata, drifter_ids)
+    if not os.path.isfile(file):
+        urllib.request.urlretrieve(url, file)
 
 
 def decode_date(t):
-    """The date format is specified in 'seconds since 1970-01-01 00:00:00' but
+    """The date format is specified as 'seconds since 1970-01-01 00:00:00' but
     the missing values are stored as -1e+34 which is not supported by the
     default parsing mechanism in xarray.
 
     This function returns replaced the missing value by NaN and returns a
     datetime instance.
-    :param t: date
-    :return: datetime
+
+    Parameters
+    ----------
+    t : array
+        Array of time values
+
+    Returns
+    -------
+    out : datetime
+        Datetime instance with the missing value replaced by NaN
     """
     nat_index = np.logical_or(np.isclose(t, -1e34), np.isnan(t))
     t[nat_index] = np.nan
@@ -235,6 +211,13 @@ def decode_date(t):
 def fill_values(var, default=np.nan):
     """Change fill values (-1e+34, inf, -inf) in var array to the value
     specified by default.
+
+    Parameters
+    ----------
+    var : array
+        Array to fill
+    default : float
+        Default value to use for fill values
     """
     missing_value = np.logical_or(np.isclose(var, -1e34), ~np.isfinite(var))
     if np.any(missing_value):
@@ -242,12 +225,21 @@ def fill_values(var, default=np.nan):
     return var
 
 
-def str_to_float(value: str, default=np.nan) -> float:
+def str_to_float(value: str, default: float = np.nan) -> float:
     """Convert a string to float, while returning the value of default if the
     string is not convertible to a float, or if it's a NaN.
 
-    :param value: str
-    :return: bool
+    Parameters
+    ----------
+    value : str
+        String to convert to float
+    default : float
+        Default value to return if the string is not convertible to float
+
+    Returns
+    -------
+    out : float
+        Float value of the string, or default if the string is not convertible to float.
     """
     try:
         fvalue = float(value)
@@ -262,25 +254,37 @@ def str_to_float(value: str, default=np.nan) -> float:
 def cut_str(value: str, max_length: int) -> np.chararray:
     """Cut a string to a specific length and return it as a numpy chararray.
 
-    Args:
-        value (str): String to cut
-        max_length (int): Length of the output
-    Returns:
-        out (np.chararray): String with max_length characters
+    Parameters
+    ----------
+    value : str
+        String to cut
+    max_length : int
+        Length of the output
+
+    Returns
+    -------
+    out : np.chararray
+        String with max_length characters
     """
     charar = np.chararray(1, max_length)
     charar[:max_length] = value
     return charar
 
 
-def drogue_presence(lost_time, time):
+def drogue_presence(lost_time, time) -> bool:
     """Create drogue status from the drogue lost time and the trajectory time.
 
-    Args:
-        lost_time: Timestamp of the drogue loss (or NaT)
-        time: Observation time
-    Returns:
-        out (bool): True if drogues and False otherwise
+    Parameters
+    ----------
+    lost_time
+        Timestamp of the drogue loss (or NaT)
+    time
+        Observation time
+
+    Returns
+    -------
+    out : bool
+        True if drogues and False otherwise
     """
     if pd.isnull(lost_time) or lost_time >= time[-1]:
         return np.ones_like(time, dtype="bool")
@@ -288,9 +292,9 @@ def drogue_presence(lost_time, time):
         return time < lost_time
 
 
-def rowsize(index: int) -> int:
+def rowsize(index: int, **kwargs) -> int:
     return xr.open_dataset(
-        os.path.join(GDP_TMP_PATH, GDP_FILENAME_PATTERN.format(id=index)),
+        os.path.join(kwargs["tmp_path"], kwargs["filename_pattern"].format(id=index)),
         decode_cf=False,
         decode_times=False,
         concat_characters=False,
@@ -298,17 +302,25 @@ def rowsize(index: int) -> int:
     ).dims["obs"]
 
 
-def preprocess(index: int) -> xr.Dataset:
-    """Extract and preprocess the Lagrangian data and attributes. This function
-    takes an identification number that can be used to: create a file or url
-    pattern or select data from a Dataframe. It then preprocess the data and
-    returns a clean xarray Dataset.
+def preprocess(index: int, **kwargs) -> xr.Dataset:
+    """Extract and preprocess the Lagrangian data and attributes.
 
-    :param index: drifter's identification number
-    :return: xr.Dataset containing the data and attributes
+    This function takes an identification number that can be used to create a
+    file or url pattern or select data from a Dataframe. It then preprocesses
+    the data and returns a clean Xarray Dataset.
+
+    Parameters
+    ----------
+    index : int
+        Drifter's identification number
+
+    Returns
+    -------
+    ds : xr.Dataset
+        Xarray Dataset containing the data and attributes
     """
     ds = xr.load_dataset(
-        os.path.join(GDP_TMP_PATH, GDP_FILENAME_PATTERN.format(id=index)),
+        os.path.join(kwargs["tmp_path"], kwargs["filename_pattern"].format(id=index)),
         decode_times=False,
         decode_coords=False,
     )
@@ -320,16 +332,22 @@ def preprocess(index: int) -> xr.Dataset:
     ds["time"].data = decode_date(np.array([ds.time.data[0]]))
 
     # convert fill values to nan
-    ds["err_lon"].data = fill_values(ds["err_lon"].data)
-    ds["err_lat"].data = fill_values(ds["err_lat"].data)
-    ds["err_ve"].data = fill_values(ds["err_ve"].data)
-    ds["err_vn"].data = fill_values(ds["err_vn"].data)
-    ds["sst"].data = fill_values(ds["sst"].data)
-    ds["sst1"].data = fill_values(ds["sst1"].data)
-    ds["sst2"].data = fill_values(ds["sst2"].data)
-    ds["err_sst"].data = fill_values(ds["err_sst"].data)
-    ds["err_sst1"].data = fill_values(ds["err_sst1"].data)
-    ds["err_sst2"].data = fill_values(ds["err_sst2"].data)
+    for var in [
+        "err_lon",
+        "err_lat",
+        "err_ve",
+        "err_vn",
+        "sst",
+        "sst1",
+        "sst2",
+        "err_sst",
+        "err_sst1",
+        "err_sst2",
+    ]:
+        try:
+            ds[var].data = fill_values(ds[var].data)
+        except KeyError:
+            warnings.warn(f"Variable {var} not found; skipping.")
 
     # fix missing values stored as str
     for var in [
@@ -345,16 +363,27 @@ def preprocess(index: int) -> xr.Dataset:
         "sst1",
         "sst2",
     ]:
-        ds[var].encoding["missing value"] = -1e-34
+        try:
+            ds[var].encoding["missing value"] = -1e-34
+        except KeyError:
+            warnings.warn(f"Variable {var} not found in upstream data; skipping.")
 
     # convert type of some variable
-    ds["ID"].data = ds["ID"].data.astype("int64")
-    ds["WMO"].data = ds["WMO"].data.astype("int32")
-    ds["expno"].data = ds["expno"].data.astype("int32")
-    ds["typedeath"].data = ds["typedeath"].data.astype("int8")
-    ds["flg_sst"].data = ds["flg_sst"].data.astype("int8")
-    ds["flg_sst1"].data = ds["flg_sst1"].data.astype("int8")
-    ds["flg_sst2"].data = ds["flg_sst2"].data.astype("int8")
+    target_dtype = {
+        "ID": "int64",
+        "WMO": "int32",
+        "expno": "int32",
+        "typedeath": "int8",
+        "flg_sst": "int8",
+        "flg_sst1": "int8",
+        "flg_sst2": "int8",
+    }
+
+    for var in target_dtype.keys():
+        if var in ds.keys():
+            ds[var].data = ds[var].data.astype(target_dtype[var])
+        else:
+            warnings.warn(f"Variable {var} not found in upstream data; skipping.")
 
     # new variables
     ds["ids"] = (["traj", "obs"], [np.repeat(ds.ID.values, ds.dims["obs"])])
@@ -366,7 +395,7 @@ def preprocess(index: int) -> xr.Dataset:
     # convert attributes to variable
     ds["location_type"] = (
         ("traj"),
-        [False if ds.location_type == "Argos" else True],
+        [False if ds.get("location_type") == "Argos" else True],
     )  # 0 for Argos, 1 for GPS
     ds["DeployingShip"] = (("traj"), cut_str(ds.DeployingShip, 20))
     ds["DeploymentStatus"] = (("traj"), cut_str(ds.DeploymentStatus, 20))
@@ -616,35 +645,13 @@ def preprocess(index: int) -> xr.Dataset:
 
     # set attributes
     for var in vars_attrs.keys():
-        ds[var].attrs = vars_attrs[var]
+        if var in ds.keys():
+            ds[var].attrs = vars_attrs[var]
+        else:
+            warnings.warn(f"Variable {var} not found in upstream data; skipping.")
     ds.attrs = attrs
 
     # rename variables
     ds = ds.rename_vars({"longitude": "lon", "latitude": "lat"})
 
     return ds
-
-
-def to_raggedarray(
-    drifter_ids: Optional[list[int]] = None, n_random_id: Optional[int] = None
-) -> RaggedArray:
-    """Download and process individual GDP hourly files and return a
-    RaggedArray instance with the data.
-
-    Args:
-        drifter_ids [list]: list of drifters to retrieve (Default: all)
-        n_random_id [int]: randomly select n drifter NetCDF files
-    Returns:
-        out [RaggedArray]: A RaggedArray instance of the requested dataset
-    """
-
-    ids = download(drifter_ids, n_random_id)
-
-    return RaggedArray.from_files(
-        indices=ids,
-        preprocess_func=preprocess,
-        name_coords=GDP_COORDS,
-        name_meta=GDP_METADATA,
-        name_data=GDP_DATA,
-        rowsize_func=rowsize,
-    )
