@@ -68,6 +68,7 @@ class RaggedArray:
         name_meta: Optional[list] = [],
         name_data: Optional[list] = [],
         rowsize_func: Optional[Callable[[int], int]] = None,
+        **kwargs,
     ):
         """Generate ragged arrays archive from a list of trajectory files
 
@@ -84,14 +85,25 @@ class RaggedArray:
         """
         # if no method is supplied, get the dimension from the preprocessing function
         rowsize_func = (
-            rowsize_func if rowsize_func else lambda i: preprocess_func(i).dims["obs"]
+            rowsize_func
+            if rowsize_func
+            else lambda i, **kwargs: preprocess_func(i, **kwargs).dims["obs"]
         )
-        rowsize = cls.number_of_observations(rowsize_func, indices)
+        rowsize = cls.number_of_observations(rowsize_func, indices, **kwargs)
         coords, metadata, data = cls.allocate(
-            preprocess_func, indices, rowsize, name_coords, name_meta, name_data
+            preprocess_func,
+            indices,
+            rowsize,
+            name_coords,
+            name_meta,
+            name_data,
+            **kwargs,
         )
         attrs_global, attrs_variables = cls.attributes(
-            preprocess_func(indices[0]), name_coords, name_meta, name_data
+            preprocess_func(indices[0], **kwargs),
+            name_coords,
+            name_meta,
+            name_data,
         )
 
         return cls(coords, metadata, data, attrs_global, attrs_variables)
@@ -168,7 +180,7 @@ class RaggedArray:
 
     @staticmethod
     def number_of_observations(
-        rowsize_func: Callable[[int], int], indices: list
+        rowsize_func: Callable[[int], int], indices: list, **kwargs
     ) -> np.array:
         """Iterate through the files and evaluate the number of observations.
 
@@ -187,7 +199,7 @@ class RaggedArray:
             desc="Retrieving the number of obs",
             ncols=80,
         ):
-            rowsize[i] = rowsize_func(index)
+            rowsize[i] = rowsize_func(index, **kwargs)
         return rowsize
 
     @staticmethod
@@ -210,7 +222,10 @@ class RaggedArray:
         # coordinates, metadata, and data
         attrs_variables = {}
         for var in name_coords + name_meta + name_data:
-            attrs_variables[var] = ds[var].attrs
+            if var in ds.keys():
+                attrs_variables[var] = ds[var].attrs
+            else:
+                warnings.warn(f"Variable {var} requested but not found; skipping.")
 
         return attrs_global, attrs_variables
 
@@ -222,6 +237,7 @@ class RaggedArray:
         name_coords: list,
         name_meta: list,
         name_data: list,
+        **kwargs,
     ) -> Tuple[dict, dict, dict]:
         """Iterate through the files and fill for the ragged array associated with coordinates, and selected metadata and data variables.
 
@@ -238,7 +254,7 @@ class RaggedArray:
         """
 
         # open one file to get dtype of variables
-        ds = preprocess_func(indices[0])
+        ds = preprocess_func(indices[0], **kwargs)
         nb_traj = len(rowsize)
         nb_obs = np.sum(rowsize).astype("int")
         index_traj = np.insert(np.cumsum(rowsize), 0, 0)
@@ -254,7 +270,10 @@ class RaggedArray:
 
         data = {}
         for var in name_data:
-            data[var] = np.zeros(nb_obs, dtype=ds[var].dtype)
+            if var in ds.keys():
+                data[var] = np.zeros(nb_obs, dtype=ds[var].dtype)
+            else:
+                warnings.warn(f"Variable {var} requested but not found; skipping.")
         ds.close()
 
         # loop and fill the ragged array
@@ -264,7 +283,7 @@ class RaggedArray:
             desc="Filling the Ragged Array",
             ncols=80,
         ):
-            with preprocess_func(index) as ds:
+            with preprocess_func(index, **kwargs) as ds:
                 size = rowsize[i]
                 oid = index_traj[i]
 
@@ -275,7 +294,12 @@ class RaggedArray:
                     metadata[var][i] = ds[var][0].data
 
                 for var in name_data:
-                    data[var][oid : oid + size] = ds[var].data
+                    if var in ds.keys():
+                        data[var][oid : oid + size] = ds[var].data
+                    else:
+                        warnings.warn(
+                            f"Variable {var} requested but not found; skipping."
+                        )
 
         return coords, metadata, data
 
